unit USmAudio;

interface

uses
   Windows, ActiveX, ComObj;

const
   IID_IMultiMediaStream    : TGUID = '{B502D1BC-9A57-11d0-8FDE-00C04FD9189D}';
   IID_IAMMultiMediaStream  : TGUID = '{BEBE595C-9A6F-11D0-8FDE-00C04FD9189D}';
   CLSID_AMMultiMediaStream : TGUID = '{49C47CE5-9BA4-11D0-8212-00C04FC32C45}';
   MSPID_PrimaryAudio       : TGUID = ( D1 : $A35FF56B; D2 : $9FDA; D3 : $11D0; D4 : ($8F,$DF,$00,$C0,$4F,$D9,$18,$9D) );
   AMMSF_ADDDEFAULTRENDERER         = $1;

type
   PAPCFUNC      = procedure( dwParam : DWORD ); stdcall;
   MSPID         = TGUID;
   TStream_Time  = Comp;

   TStream_Type  =
   (
      STREAMTYPE_READ,
      STREAMTYPE_WRITE,
      STREAMTYPE_TRANSFORM
   );

   TStream_State =
   (
      STREAMSTATE_STOP,
      STREAMSTATE_RUN
   );

   IMediaStream       = IUnknown;
   IMediaStreamFilter = IUnknown;
   IGraphBuilder      = IUnknown;

   IMultiMediaStream  = interface( IUnknown )
      ['{B502D1BC-9A57-11d0-8FDE-00C04FD9189D}']
      function GetInformation( var pdwFlags : DWORD; var pStreamType : TStream_Type ) :  HRESULT; stdcall;
      function GetMediaStream( const idPurpose : MSPID; out ppMediaStream : IMediaStream ) : HRESULT; stdcall;
      function EnumMediaStreams( Index : Longint; out ppMediaStream : IMediaStream ) : HRESULT; stdcall;
      function GetState( var pCurrentState : TStream_State ) : HRESULT; stdcall;
      function SetState( NewState : TStream_State ) : HRESULT; stdcall;
      function GetTime( var pCurrentTime : TStream_Time ) : HRESULT; stdcall;
      function GetDuration( var pDuration : TStream_Time ) : HRESULT; stdcall;
      function Seek( SeekTime : TStream_Time ) : HRESULT; stdcall;
      function GetEndOfStreamEventHandle( var phEOS : THandle ) : HRESULT; stdcall;
   end;


   IAMMultiMediaStream = interface( IMultiMediaStream )
      ['{BEBE595C-9A6F-11D0-8FDE-00C04FD9189D}']
      function Initialize( StreamType : TStream_Type; dwFlags : DWORD; pFilterGraph : IGraphBuilder) : HRESULT; stdcall;
      function GetFilterGraph( out ppGraphBuilder : IGraphBuilder ) : HRESULT; stdcall;
      function GetFilter( out ppFilter : IMediaStreamFilter ) : HRESULT; stdcall;
      function AddMediaStream( pStreamObject : IUnknown; const PurposeId : MSPID; dwFlags : DWORD; out ppNewStream : IMediaStream ) : HRESULT; stdcall;
      function OpenFile( pszFileName : LPCWSTR; dwFlags : DWORD ) : HRESULT; stdcall;
      function OpenMoniker( pCtx : IBindCtx; pMoniker : IMoniker; dwFlags : DWORD ) : HRESULT; stdcall;
      function Render( dwFlags : DWORD) : HRESULT; stdcall;
  end;

type

   TSmAudio = class
      constructor Create;
      procedure   Free;

   private
      IsOpened   : Boolean;
      DMMStream  : IMultiMediaStream;

   public
      procedure Open( FileName : WideString );
      procedure Play;
      procedure Pause;
      procedure Stop;
      procedure Close;
      procedure Rewind( Position : LongInt );
      procedure Forward( Position : LongInt );
      function  GetLen : LongInt;
      function  GetPos : LongInt;
      procedure SetPos( Position : LongInt );

      property  Length   : Longint read GetLen;
      property  Position : Longint read GetPos  write SetPos;
      
   end;

implementation

constructor TSmAudio.Create;
begin
   IsOpened := FALSE;
end;

procedure TSmAudio.Free;
begin
   Close;
end;

procedure TSmAudio.Open( FileName : WideString );
var
   DAMStream : IAMMultiMediaStream;
begin
   Pointer( DAMStream ) := nil;

   try
      DAMStream := IAMMultiMediaStream( CreateComObject( CLSID_AMMultiMediaStream ) );
      if not Assigned( DAMStream ) then
         Exit;

      if DAMStream.Initialize( STREAMTYPE_READ, 0, nil ) <> S_OK then
         Exit;

      DAMStream.AddMediaStream( nil, MSPID_PrimaryAudio, AMMSF_ADDDEFAULTRENDERER, IMediaStream( nil^ ) );

      if DAMStream.OpenFile( PWideChar( FileName ), 0 ) <> S_OK then
         Exit;

      DAMStream.QueryInterface( IID_IMultiMediaStream, DMMStream );

      IsOpened := TRUE;

   finally
      if Assigned( DAMStream ) then
         DAMStream := nil;

      if not IsOpened then begin
         if Assigned( DMMStream ) then
            DMMStream := nil;
      end;

   end;
end;

procedure TSmAudio.Play;
begin
   if IsOpened then
      DMMStream.SetState( STREAMSTATE_RUN );
end;

procedure TSmAudio.Stop;
begin
   if IsOpened then
      DMMStream.SetState( STREAMSTATE_STOP );
end;

procedure TSmAudio.Pause;
begin
   if IsOpened then
      DMMStream.SetState( STREAMSTATE_STOP );
end;

procedure TSmAudio.Close;
begin
   if IsOpened then begin
      DMMStream.SetState( STREAMSTATE_STOP );
      DMMStream  := nil;
      IsOpened   := FALSE;
   end;
end;

procedure TSmAudio.Forward( Position : LongInt );
var
   CurrentTime : TStream_Time;
begin
   DMMStream.GetTime( CurrentTime );
   DMMStream.Seek( CurrentTime + Position * 10000.0 );
end;

procedure TSmAudio.Rewind( Position : LongInt );
var
   CurrentTime : TStream_Time;
begin
   DMMStream.GetTime( CurrentTime );
   DMMStream.Seek( CurrentTime - Position * 10000.0 );
end;

function TSmAudio.GetLen: LongInt;
begin
//
end;

function TSmAudio.GetPos: LongInt;
begin
//
end;

procedure TSmAudio.SetPos( Position : LongInt );
begin
   DMMStream.Seek( Position * 10000.0 );
end;

end.
