unit USmSAMI;

interface

uses
   Windows, ActiveX;

const
   IID_IGraphBuilder : TGUID = '{56A868A9-0AD4-11CE-B03A-0020AF0BA770}';
   IID_IMediaControl : TGUID = (D1:$56A868B1;D2:$0AD4;D3:$11CE;D4:($B0,$3A,$00,$20,$AF,$0B,$A7,$70));
   IID_IMediaEventEx : TGUID = (D1:$56A868C0;D2:$0AD4;D3:$11CE;D4:($B0,$3A,$00,$20,$AF,$0B,$A7,$70));
   CLSID_FilterGraph : TGUID = (D1:$E436EBB3;D2:$524F;D3:$11CE;D4:($9F,$53,$00,$20,$AF,$0B,$A7,$70));

   EC_COMPLETE               = $01;
   EC_USERABORT              = $02;
   EC_ERRORABORT             = $03;

   EC_OLE_EVENT              = $18;

type
   TAM_Media_Type = record
      majortype            : TGUID;
      subtype              : TGUID;
      bFixedSizeSamples    : BOOL;
      bTemporalCompression : BOOL;
      lSampleSize          : ULONG;
      formattype           : TGUID;
      pUnk                 : IUnknown;
      cbFormat             : ULONG;
      pbFormat             : Pointer;
   end;

   IPin          = IUnknown;
   IBaseFilter   = IUnknown;
   IEnumFilters  = IUnknown;

   IFilterGraph = interface( IUnknown )
      ['{56A8689F-0AD4-11CE-B03A-0020AF0BA770}']
      function AddFilter        ( pFilter           : IBaseFilter;
                                  pName             : LPCWSTR        ) : HRESULT; stdcall;
      function RemoveFilter     ( pFilter           : IBaseFilter    ) : HRESULT; stdcall;
      function EnumFilters      ( out ppEnum        : IEnumFilters   ) : HRESULT; stdcall;
      function FindFilterByName ( pName             : LPCWSTR;
                                  out ppFilter      : IBaseFilter    ) : HRESULT; stdcall;
      function ConnectDirect    ( ppinOut           : IPin;
                                  ppinIn            : IPin;
                                  const pmt         : TAM_Media_Type ) : HRESULT; stdcall;
      function Reconnect        ( ppin              : IPin           ) : HRESULT; stdcall;
      function Disconnect       ( ppin              : IPin           ) : HRESULT; stdcall;
      function SetDefaultSyncSource                                    : HRESULT; stdcall;
   end;

   IGraphBuilder = interface( IFilterGraph )
      ['{56A868A9-0AD4-11CE-B03A-0020AF0BA770}']
      function Connect          ( ppinOut           : IPin;
                                  ppinIn            : IPin        ) : HRESULT; stdcall;
      function Render           ( ppinOut           : IPin        ) : HRESULT; stdcall;
      function RenderFile       ( lpcwstrFile       : LPCWSTR;
                                  lpcwstrPlayList   : LPCWSTR     ) : HRESULT; stdcall;
      function AddSourceFilter  ( lpcwstrFileName   : LPCWSTR;
                                  lpcwstrFilterName : LPCWSTR;
                                  out ppFilter      : IBaseFilter ) : HRESULT; stdcall;
      function SetLogFile       ( hFile             : THandle     ) : HRESULT; stdcall;
      function Abort                                                : HRESULT; stdcall;
      function ShouldOperationContinue                              : HRESULT; stdcall;
   end;

   IMediaControl = interface( IDispatch )
      ['{56A868B1-0AD4-11CE-B03A-0020AF0BA770}']
      procedure Run;                                                 safecall;
      procedure Pause;                                               safecall;
      procedure Stop;                                                safecall;
      procedure GetState        ( msTimeout         : Integer;
                                  out pfs           : Integer     ); safecall;
      procedure RenderFile      ( const strFilename : WideString  ); safecall;
      procedure AddSourceFilter ( const strFilename : WideString;
                                  out ppUnk         : IDispatch   ); safecall;
      function  Get_FilterCollection                : IDispatch;     safecall;
      function  Get_RegFilterCollection             : IDispatch;     safecall;
      procedure StopWhenReady;                                       safecall;
      property  FilterCollection                    : IDispatch      read Get_FilterCollection;
      property  RegFilterCollection                 : IDispatch      read Get_RegFilterCollection;
   end;

   OAEVENT = Longint;
   OAHWND  = Longint;

   IMediaEvent = interface( IDispatch )
      ['{56A868B6-0AD4-11CE-B03A-0020AF0BA770}']
      function  GetEventHandle         ( var hEvent     : OAEVENT ) : HRESULT; stdcall;
      function  GetEvent               ( var lEventCode : Longint;
                                         var lParam1    : Longint;
                                         var lParam2    : Longint;
                                         msTimeout      : Longint ) : HRESULT; stdcall;
      function  WaitForCompletion      ( msTimeout      : Longint;
                                         var pEvCode    : Longint ) : HRESULT; stdcall;
      function  CancelDefaultHandling  ( lEvCode        : Longint ) : HRESULT; stdcall;
      function  RestoreDefaultHandling ( lEvCode        : Longint ) : HRESULT; stdcall;
      function  FreeEventParams        ( lEvCode        : Longint;
                                         lParam1        : Longint;
                                         lParam2        : Longint ) : HRESULT; stdcall;
   end;

   IMediaEventEx = interface( IMediaEvent )
      ['{56A868C0-0AD4-11CE-B03A-0020AF0BA770}']
      function  SetNotifyWindow        ( hwnd                 : OAHWND;
                                         lMsg                 : Longint;
                                         lInstanceData        : Longint ) : HRESULT; stdcall;
      function  SetNotifyFlags         ( lNoNotifyFlags       : Longint ) : HRESULT; stdcall;
      function  GetNotifyFlags         ( var lplNoNotifyFlags : Longint ) : HRESULT; stdcall;
   end;

type
   TSmSAMI = class
      constructor Create;
      procedure   Free;

   private

      pGraph        : IGraphBuilder;
      pMediaControl : IMediaControl;
      pEv           : IMediaEventEx;

      procedure   Initialize;
      procedure   Finalize;
      procedure   GetRawCaption ( const SourStr : String;
                                  out   DestStr : String );

   public
      IsOpened      : Boolean;

      procedure   Open          ( FileName      : String );
      procedure   Run;
      function    GetCaption    ( var Caption   : String;
                                  AllCaption    : Boolean ) : Boolean;
      procedure   Close;
      procedure   Pause;

   end;

implementation

constructor TSmSAMI.Create;
begin
   IsOpened := FALSE;
   CoInitialize( nil );
   Initialize;
end;

procedure   TSmSAMI.Free;
begin
   Finalize;
   CoUninitialize();
end;

procedure   TSmSAMI.Initialize;
begin
   //  ٲ.. Direct X SDK help    ϱ..
   CoCreateInstance( CLSID_FilterGraph, nil, CLSCTX_INPROC, IID_IGraphBuilder, pGraph );
   pGraph.QueryInterface( IID_IMediaControl, pMediaControl);
   pGraph.QueryInterface( IID_IMediaEventEx, pEv);
end;

procedure   TSmSAMI.Finalize;
begin
   pEv           := nil;
   pMediaControl := nil;
   pGraph        := nil;
end;

procedure   TSmSAMI.Open( FileName : String );
var
   hr          : HRESULT;
   wFileName   : array[0..256] of WideChar;
begin
   if not Assigned( pGraph ) then
      Exit;

   StringToWideChar( FileName, wFileName, sizeof( wFileName ) );
   hr := pGraph.RenderFile( @wFileName, nil );

   if Failed( hr ) then
      Exit;

   IsOpened := TRUE;
end;

procedure   TSmSAMI.Run;
begin
   if not Assigned( pMediaControl ) then
      Exit;

   if not IsOpened then
      Exit;

   pMediaControl.Run;
end;

procedure   TSmSAMI.GetRawCaption( const SourStr : String; out DestStr : String );
const
   TAG_BEGIN = '<';
   TAG_END   = '>';
var
   TagBegin, TagEnd : Integer;
begin
   DestStr := SourStr;

   //  ļ
   if Pos( TAG_END, SourStr ) > 0 then begin

      while ( TRUE ) do begin

         TagBegin := Pos( TAG_BEGIN, DestStr );
         TagEnd   := Pos( TAG_END,   DestStr );

         if ( TagBegin = 0 ) or ( TagEnd = 0 ) then
            break;

         if ( TagBegin < TagEnd ) then begin
            Delete( DestStr, TagBegin, TagEnd - TagBegin + 1 );
         end
         else begin
            DestStr := Copy( DestStr, TagEnd+1, 65000 );
         end;
      end;
   end;
end;

function    TSmSAMI.GetCaption( var Caption : String; AllCaption : Boolean ) : Boolean;
var
   evCode, lParam1, lParam2 : Integer;
begin
   GetCaption := FALSE;
   Caption    := '';

   if not Assigned( pEv ) then
      Exit;

   if not IsOpened then
      Exit;

   pEv.GetEvent( evCode, lParam1, lParam2, 100 );

   if evCode = EC_OLE_EVENT then begin

      if AllCaption then begin
         WideCharToStrVar( PWideChar( lParam2 ), Caption );
      end
      else begin
         GetRawCaption( WideCharToString( PWideChar( lParam2 ) ), Caption );
      end;

      if Caption = '&nbsp;' then
         Caption := ' ';
   end;

   pEv.FreeEventParams( evCode, lParam1, lParam2 );

   GetCaption := not (( evCode = EC_USERABORT ) or ( evCode = EC_COMPLETE ) or ( evCode = EC_ERRORABORT ));
end;

procedure   TSmSAMI.Close;
begin
   pMediaControl.Stop;
end;

procedure   TSmSAMI.Pause;
begin
   pMediaControl.Pause;
end;

end.

